@@ -87,7 +87,7 @@ DataClass::~DataClass(){
 }
 		 	
 double DataClass::calculatePSI(void ){
-	unsigned long *QMAT = this->qMAT;
+	double *QMAT = this->qMAT;
 	unsigned int* bn = this->BN ;
 	unsigned int n = this->ncol;
 	unsigned int bmax = this->nrow;
@@ -121,15 +121,18 @@ double DataClass::calculatePSI(void ){
 
 void DataClass::R2Q(){
 	//Convert R matrix to Q matrix (i.e. Q_1 = R_2-R_1-1):
+	//cout << "Q:\n";
 	unsigned int i=0,j=0;
 	for (j = 0; j < this->ncol; j++){
 		for (i = 0; i < (this->BN[j]-1); i++){
-			this->qMAT[j*this->nrow+i] = this->rMAT[j*this->nrow+i+1]-this->rMAT[j*this->nrow+i] -1;
+			this->qMAT[j*this->nrow+i] = this->rMAT[j*this->nrow+i+1]-this->rMAT[j*this->nrow+i]-1;
+			//cout << "\t" << this->qMAT[j*this->nrow+i];
 		}
+		//cout << "\n";
 	}
 }
 
-
+/*INTACT FUCNTION
 void DataClass::S2R(){
 	//Convert to int
 	unsigned long count =0,i=0,j=0;
@@ -145,15 +148,129 @@ void DataClass::S2R(){
 	}
 	sort(pKVP,pKVP+(this->nrow*this->ncol),predicate_k_v_pair);//And sort based on the values
 
-	for (i = 0; i < (this->nrow*this->ncol - nInf); i++){
-		this->rMAT[pKVP[i].lPosition] = i+1 ;  //Assign the ranks based on the sorted key value pairs
+	i = 0;
+	unsigned long from = 0, to=0;
+	while (i < (this->nrow*this->ncol - nInf)){
+		from = i;
+		while (i < (this->nrow*this->ncol - nInf-1) & pKVP[i].dValue==pKVP[i+1].dValue) {i++;} //While tied value
+		to = i;
+
+		//cout << "from = " << from<<"\t to = " << to << endl;
+		for ( i= from; i <= to; i++) {
+			this->rMAT[pKVP[i].lPosition] = 1+ (from+to)/2 ;  //Assign the ranks based on the sorted key value pairs
+		//	cout << 1+(from+to)/2<< endl;
+		}	
 	}
+
 	for (i = (this->nrow*this->ncol - nInf); i < (this->nrow*this->ncol ); i++){
 		this->rMAT[pKVP[i].lPosition] = std::numeric_limits<unsigned long>::max();  //Assign the ranks based on the sorted key value pairs
 	}
 	free(pKVP);
 }
 
+*/
+
+
+void DataClass::S2R(){
+	//Convert to int
+	unsigned long count =0,i=0,j=0,k=0,v=0;
+	unsigned long nInf = 0;
+	double l =0.0;
+
+	lKEY_dVALUE_PAIR* pKVP = (lKEY_dVALUE_PAIR*)malloc(sizeof(lKEY_dVALUE_PAIR) * (this->ncol* this->nrow)); //a vector (for each element in the matrix) of key value pairs of doubles 
+
+	//First sort all values in the whole matrix	to obtain rank values		
+	for (j = 0; j < this->ncol; j++){
+		for (i = 0; i < this->nrow; i++){
+			pKVP[count].lPosition = count; //Set each key value pair
+			pKVP[count++].dValue = *(sMAT+j* this->nrow+i);
+			if (*(sMAT+j* this->nrow+i) == std::numeric_limits<double>::infinity() ) {nInf++;}
+		}
+	}
+	 
+	sort(pKVP,pKVP+(this->nrow*this->ncol),predicate_k_v_pair);
+
+	//for (j = 0; j < this->nrow  * this->ncol; j++){
+	//	cout << "Value: " << pKVP[j].dValue <<"\torder: " << j << "\tposition :"<< pKVP[j].lPosition << "\tsubject :"<< (long)pKVP[j].lPosition/this->nrow << "\n";
+	//}
+	
+	//Next loop  through the sorted values except the infinities which are at the end
+	i = 0;
+	unsigned long from = 0, to = 0, from2 = 0, to2 = 0;
+ 	
+	
+	for (i=0; i < (this->nrow*this->ncol - nInf);i++){
+		//And detect stretches of tied values
+
+		from = i;to=i;
+		while (i < (this->nrow*this->ncol - nInf-1)){
+		 	if (pKVP[i].dValue!=pKVP[i+1].dValue){break;} 
+			i++; //While tied value
+		}
+		to = i;
+
+
+		//in case of no tied values:
+		if ((to-from)==0) { 
+			//cout << "\nAssessing NON-tied value : " << pKVP[from].dValue << "(From: " << from << "   to:"<<to << ")"<<endl;
+			this->rMAT[pKVP[from].lPosition] = 1 + from ;  //Assign the ranks based on the sorted key value pairs
+		} else { //otherwise, if case of tied value:
+
+			double mu = (2+to+from)/2.0;
+		
+			//cout << "\nAssessing tied value : " << pKVP[from].dValue << "(From: " << from << "   to:"<<to << ")"<<endl;
+			//cout <<"\t mu=" << mu << endl;
+
+			//Make sure that the tied values are sorted by subject.
+			
+			for (j = from; j <= to; j++){
+				pKVP[j].dValue = pKVP[j].lPosition ;
+			}
+			sort(&(pKVP[from]),pKVP+(to),predicate_k_v_pair);
+
+			//for (j = from; j <= to; j++){
+			//   cout << "Value: " << pKVP[j].dValue <<"\torder: " << j << "\tposition :"<< pKVP[j].lPosition << "\tsubject :"<< (long)pKVP[j].lPosition/this->nrow << "\n";
+			//}
+	
+			long curSubj;j=from;		
+			//Loop through them 
+			while(j<=to){
+				//And determine the begin and end of each subject.
+				curSubj = (long) (pKVP[j].lPosition / this->nrow);
+				from2 = j;	
+				while ( (j<=to) & (curSubj == ((long) (pKVP[j].lPosition / this->nrow)))){j++;}
+				to2 = j;
+
+				//Based on the number inside and ouside the subject, calculate and set the expected ranking
+				k= to2-from2;
+				v = 1+to-from-k;
+				//cout << "\tFrom2: " << from2 << "   to2:"<<to2 << endl;
+				//cout << "\t"<<k << " tie(s) within subject" << curSubj << " and " << v << "ouside subject"<< endl;
+				count = 0;
+				for (l = 1-(k+1)/2.0; l<=(k -(k+1)/2.0); l++){
+					//cout <<"\t\t\t"<<count <<": adding " <<mu+l*(1.0+(v)/(k+1.0))<<" to position "<<pKVP[from2+count].lPosition<<endl;
+					this->rMAT[pKVP[from2+count].lPosition] = mu+l*(1.0+(v)/(k+1.0)) ;		
+					count++;
+						
+				}
+				
+			}
+	
+		} 
+		
+		
+		
+	}
+
+	for (i = (this->nrow*this->ncol - nInf); i < (this->nrow*this->ncol ); i++){
+		this->rMAT[pKVP[i].lPosition] = std::numeric_limits<double>::infinity();  //Assign the missing values as inifinies.
+	}
+
+	 
+	free(pKVP);
+}
+
+
 void DataClass::orderPerSubject(){
 	//pmat1 is a pointer to double matrix, In the rows are the observers (microarrays), in the columns the subjects (patients)
 	unsigned int j;	
@@ -162,7 +279,7 @@ void DataClass::orderPerSubject(){
 	} 
 }
 
-
+/*
 void DataClass::BN_from_R(){
 	unsigned long i,j;
 	this->T = 0;
@@ -179,17 +296,38 @@ void DataClass::BN_from_R(){
 		this->omega += this->BN[j]*(this->BN[j]-1) * (this->T-this->BN[j]);
 	} 
 	//cout << "omega = " << this->omega << endl;
+}*/
+
+void DataClass::BN_from_S(){
+	unsigned long i,j;
+	this->T = 0;
+	this->omega = 0;
+	//Determine BN (= vector of length b indicating the number of known patients with 1..b measurements)  and T (=total number of pivots) and omega (=b*(b-1)*(t-b)
+	for (j = 0; j < this->ncol; j++){
+		i = this->nrow-1;
+		while ((*(this->sMAT + j * this->nrow + i))==std::numeric_limits<double>::infinity()){ i--;} 
+		this->BN[j] = i+1; //Set the number of real values for patient j
+		(this->T)+=this->BN[j]; //And the number of pivots
+	} 
+	//Now that T is known , we can determine omega
+	for (j = 0; j < this->ncol; j++){
+		this->omega += this->BN[j]*(this->BN[j]-1) * (this->T-this->BN[j]);
+	} 
+	//cout << "omega = " << this->omega << endl;
 }
 
 void DataClass::preprocess( void ){
 	//Order each column
 	this->orderPerSubject();
 		
+	//Determine BN (= vector of length b indicating the number of known patients with 1..b measurments)  and T (=total number of pivots irrespective of b: t_b +b = T);
+	this->BN_from_S();
+	
 	//Convert to int
 	this->S2R();
 
 	//Determine BN (= vector of length b indicating the number of known patients with 1..b measurments)  and T (=total number of pivots irrespective of b: t_b +b = T);
-	this->BN_from_R();
+	//this->BN_from_R();
 	 
 	this->R2Q();
 }
@@ -206,8 +344,8 @@ DataClass::DataClass(double* pmat1, unsigned int n, unsigned int maxB){
 	this->BN = (unsigned int*)malloc(sizeof(unsigned int) * this->ncol);
 
 	this->sMAT = (double*)malloc(sizeof(double) * (this->ncol*this->nrow));
-	this->qMAT = (unsigned long*)malloc(sizeof(unsigned long) * (this->ncol*this->nrow));
-	this->rMAT = (unsigned long*)malloc(sizeof(unsigned long) * (this->ncol*this->nrow));	
+	this->qMAT = (double*)malloc(sizeof(double) * (this->ncol*this->nrow));
+	this->rMAT = (double*)malloc(sizeof(double) * (this->ncol*this->nrow));	
 	
 	for (unsigned int i = 0; i < (this->ncol*this->nrow); i++){		this->sMAT[i] = pmat1[i];	}
 
@@ -237,14 +375,14 @@ DataClass::DataClass (const DataClass &obj) {
 	}
 
 	if (obj.qMAT != NULL) {	
-		this->qMAT = (unsigned long*)malloc(sizeof(unsigned long) * (this->ncol*this->nrow));
+		this->qMAT = (double*)malloc(sizeof(double) * (this->ncol*this->nrow));
 		for (unsigned int i = 0; i < (this->ncol*this->nrow); i++){
 			this->qMAT [i] = obj.qMAT[i];
 		}
 	}
 
 	if (obj.rMAT != NULL) {	
-		this->rMAT = (unsigned long*)malloc(sizeof(unsigned long) * (this->ncol*this->nrow));
+		this->rMAT = (double*)malloc(sizeof(double) * (this->ncol*this->nrow));
 		for (unsigned int i = 0; i < (this->ncol*this->nrow); i++){
 			this->rMAT [i] = obj.rMAT[i];
 		}
